/// <reference types="node" />
import { IncomingMessage, OutgoingHttpHeaders, OutgoingMessage } from 'http';
import { HttpMethod } from '@marblejs/core';
import { Readable } from 'stream';
export declare type ServerApp = (req: IncomingMessage, res: OutgoingMessage) => void;
export declare type Logger = (message: string) => void;
export interface ServerProxyRequest {
    method: HttpMethod;
    path: string;
    host?: string;
    protocol?: string;
    headers?: OutgoingHttpHeaders;
    body?: Buffer | Readable;
}
export interface ServerProxyResponse {
    statusCode: number;
    statusMessage?: string;
    body?: Buffer;
    headers: Record<string, string[]>;
}
export declare abstract class ServerProxy<ProxyRequest, ProxyResponse> {
    private app;
    protected log: Logger;
    private readonly server;
    private listening;
    private socketPath;
    constructor(app: ServerApp);
    close(): void;
    abstract normalizeRequest(proxyRequest: ProxyRequest): ServerProxyRequest | Promise<ServerProxyRequest>;
    abstract normalizeResponse(serverProxyResponse: ServerProxyResponse): ProxyResponse | Promise<ProxyResponse>;
    abstract normalizeError(error: Error): ProxyResponse | Promise<ProxyResponse>;
    handle(proxyRequest: ProxyRequest): Promise<ProxyResponse>;
    /** Emits an error to server for testing purposes */
    emitError(error: Error): void;
    private logWithTag;
    private sendToServer;
    private startServer;
}
