"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const http_1 = require("http");
const serverProxy_helpers_1 = require("./serverProxy.helpers");
const core_1 = require("@marblejs/core");
const _internal_1 = require("@marblejs/core/dist/+internal");
class ServerProxy {
    constructor(app) {
        this.app = app;
        /* istanbul ignore next */
        this.log = () => void 0;
        this.listening = false;
        this.startServer = () => this.server.listen(this.socketPath);
        this.socketPath = serverProxy_helpers_1.makeSocketPath();
        this.server = http_1.createServer(this.app);
        this.server
            .on('close', () => {
            this.listening = false;
            this.logWithTag('closed');
        })
            .on('error', (error) => {
            this.logWithTag(error.toString());
            if (error.code === 'EADDRINUSE') {
                this.socketPath = serverProxy_helpers_1.makeSocketPath();
                this.server.close(() => this.startServer());
                return;
            }
        })
            .on('listening', () => {
            this.listening = true;
            this.logWithTag('listening');
        });
    }
    close() {
        this.server.close();
    }
    handle(proxyRequest) {
        return __awaiter(this, void 0, void 0, function* () {
            const serverProxyRequest = yield Promise.resolve(this.normalizeRequest(proxyRequest));
            return yield new Promise((resolve) => {
                if (this.listening) {
                    this.sendToServer(serverProxyRequest, resolve);
                }
                else {
                    this.server.on('listening', () => this.sendToServer(serverProxyRequest, resolve));
                    this.startServer();
                }
            });
        });
    }
    /** Emits an error to server for testing purposes */
    emitError(error) {
        this.server.emit('error', error);
    }
    logWithTag(message) {
        this.log(`[ServerProxy ${this.socketPath}] ${message}`);
    }
    sendToServer(serverProxyRequest, resolve) {
        try {
            const { headers = {}, body, host, path, method, protocol } = serverProxyRequest;
            const req = http_1.request({
                headers,
                path,
                method,
                host,
                protocol,
                socketPath: this.socketPath,
            }, (response) => {
                const bufferChunks = [];
                response
                    .on('data', chunk => bufferChunks.push(chunk))
                    .on('end', () => {
                    /* istanbul ignore next: Status code is always present in Marble, but TS requires to specify default */
                    const { statusCode = core_1.HttpStatus.OK, statusMessage, headers, } = response;
                    resolve(this.normalizeResponse({
                        statusCode,
                        statusMessage,
                        headers: serverProxy_helpers_1.normalizeHeaders(headers),
                        body: Buffer.concat(bufferChunks),
                    }));
                });
            }).on('error', error => resolve(this.normalizeError(error)));
            if (body) {
                if (_internal_1.isStream(body)) {
                    body.pipe(req);
                }
                else {
                    req.write(body);
                }
            }
            req.end();
        }
        catch (error) {
            this.logWithTag(error.toString());
            resolve(this.normalizeError(error));
        }
    }
    ;
}
exports.ServerProxy = ServerProxy;
